#!/usr/bin/env python3

import i3ipc
from sway_auto_floating.shared import load_state, save_state, key_for

def apply_remembered(conn, container, state):
    k = key_for(conn, container)

    if state.get(k):
        container.command('floating enable')

def main():
    conn = i3ipc.Connection()
    state = load_state()

    # Apply to existing windows at startup
    for container in conn.get_tree().leaves():
        apply_remembered(conn, container, state)

    def on_window(conn, e):
        state = load_state()
        container = e.container
        apply_remembered(conn, container, state)

    # container.name is empty on WINDOW_NEW, hence using WINDOW_TITLE
    conn.on(i3ipc.Event.WINDOW_TITLE, on_window)

    conn.main()

if __name__ == "__main__":
    main()



import json
from pathlib import Path

STATE_PATH = Path.home() / ".config/sway_auto_floating/floating-state.json"

def load_state():
    try:
        return json.loads(STATE_PATH.read_text())
    except Exception:
        return {}

def save_state(state):
    STATE_PATH.parent.mkdir(parents=True, exist_ok=True)
    tmp = STATE_PATH.with_suffix(".tmp")
    tmp.write_text(json.dumps(state, indent=2, sort_keys=True))
    tmp.replace(STATE_PATH)

def key_for(conn, container):
    base_key = f"wayland::{container.app_id}"

    # Check if another window with same app_id exists
    all_windows = conn.get_tree().leaves()

    for w in all_windows:
        # If it does, use more specific key
        if w.id != container.id and w.app_id == container.app_id:
            return f"{base_key}::{container.name[:50]}"

    return base_key



#!/usr/bin/env python3

import i3ipc
from sway_auto_floating.shared import load_state, save_state, key_for

def main():
    conn = i3ipc.Connection()

    focused = conn.get_tree().find_focused()

    focused.command('floating toggle')

    state = load_state()

    k = key_for(conn, focused)

    is_floating = (focused.floating in ("auto_off"))

    if is_floating:
        state[k] = True
    elif k in state:
        del state[k]

    save_state(state)

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name='sway_auto_floating',
    version='1.0',
    description='Remember floating state of Sway windows and automatically set it when a new window of the same app is created.',
    author='Artem Avetisyan',
    packages=find_packages(),
    license='MIT',
    url='https://github.com/artemave/sway_auto_floating',
    include_package_data=True,
    install_requires=[
        'i3ipc>=2.2.1',
    ],
    entry_points={
        'console_scripts': [
            'sway_toggle_floating = sway_auto_floating.toggle_floating:main',
            'sway_auto_floating = sway_auto_floating.auto_floating:main',
        ],
    },
)
